@@ -47,15 +47,16 @@ using std::cout;
 using std::endl;
 
 //*******************************************************************************
-AudioInterface::AudioInterface(JackTrip* jacktrip, int NumInChans, int NumOutChans,
+AudioInterface::AudioInterface(QVarLengthArray<int> InputChans,
+                               QVarLengthArray<int> OutputChans,
+                               inputMixModeT InputMixMode,
 #ifdef WAIR  // wair
                                int NumNetRevChans,
 #endif  // endwhere
                                audioBitResolutionT AudioBitResolution,
-                               bool processWithNetwork)
-    : mJackTrip(jacktrip)
-    , mNumInChans(NumInChans)
-    , mNumOutChans(NumOutChans)
+                               bool processWithNetwork, JackTrip* jacktrip)
+    : mInputChans(InputChans)
+    , mOutputChans(OutputChans)
     ,
 #ifdef WAIR  // WAIR
     mNumNetRevChans(NumNetRevChans)
@@ -69,40 +70,57 @@ AudioInterface::AudioInterface(JackTrip* jacktrip, int NumInChans, int NumOutCha
     , mAudioOutputPacket(NULL)
     , mLoopBack(false)
     , mProcessWithNetwork(processWithNetwork)
+    , mJackTrip(jacktrip)
+    , mInputMixMode(InputMixMode)
     , mProcessingAudio(false)
 {
 #ifndef WAIR
     // cc
     // Initialize and assign memory for ProcessPlugins Buffers
-    mInProcessBuffer.resize(mNumInChans);
-    mOutProcessBuffer.resize(mNumOutChans);
+    int monitorChans = std::min(mInputChans.size(), mOutputChans.size());
+    mInProcessBuffer.resize(mInputChans.size());
+    mOutProcessBuffer.resize(mOutputChans.size());
+    mMonProcessBuffer.resize(monitorChans);
     // Set pointer to NULL
-    for (int i = 0; i < mNumInChans; i++) {
+    for (int i = 0; i < mInProcessBuffer.size(); i++) {
         mInProcessBuffer[i] = NULL;
     }
-    for (int i = 0; i < mNumOutChans; i++) {
+    for (int i = 0; i < mOutProcessBuffer.size(); i++) {
         mOutProcessBuffer[i] = NULL;
     }
+    for (int i = 0; i < monitorChans; i++) {
+        mMonProcessBuffer[i] = NULL;
+    }
 #else   // WAIR
-    int iCnt = (mNumInChans > mNumNetRevChans) ? mNumInChans : mNumNetRevChans;
-    int oCnt = (mNumOutChans > mNumNetRevChans) ? mNumOutChans : mNumNetRevChans;
-    int aCnt = (mNumNetRevChans) ? mNumInChans : 0;
+    int iCnt =
+        (mInputChans.size() > mNumNetRevChans) ? mInputChans.size() : mNumNetRevChans;
+    int oCnt =
+        (mOutputChans.size() > mNumNetRevChans) ? mOutputChans.size() : mNumNetRevChans;
+    int aCnt = (mNumNetRevChans) ? mInputChans.size() : 0;
+    int mCnt = std::min(iCnt, oCnt);
     for (int i = 0; i < iCnt; i++) {
         mInProcessBuffer[i] = NULL;
     }
     for (int i = 0; i < oCnt; i++) {
         mOutProcessBuffer[i] = NULL;
     }
+    for (int i = 0; i < mCnt; i++) {
+        mMonProcessBuffer[i] = NULL;
+    }
     for (int i = 0; i < aCnt; i++) {
         mAPInBuffer[i] = NULL;
     }
 #endif  // endwhere
 
-    mInBufCopy.resize(mNumInChans);
-    for (int i = 0; i < mNumInChans; i++) {
+    mInBufCopy.resize(mInputChans.size());
+    for (int i = 0; i < mInputChans.size(); i++) {
         mInBufCopy[i] =
             new sample_t[MAX_AUDIO_BUFFER_SIZE];  // required for processing audio input
     }
+
+    // Not used in this class but may be needed by subclasses
+    mNumInChans  = mInputChans.size();
+    mNumOutChans = mOutputChans.size();
 }
 
 //*******************************************************************************
@@ -111,35 +129,40 @@ AudioInterface::~AudioInterface()
     delete[] mAudioInputPacket;
     delete[] mAudioOutputPacket;
 #ifndef WAIR  // NOT WAIR:
-    for (int i = 0; i < mNumInChans; i++) {
+    for (int i = 0; i < mInProcessBuffer.size(); i++) {
         delete[] mInProcessBuffer[i];
     }
 
-    for (int i = 0; i < mNumOutChans; i++) {
+    for (int i = 0; i < mOutProcessBuffer.size(); i++) {
         delete[] mOutProcessBuffer[i];
     }
+    for (int i = 0; i < mMonProcessBuffer.size(); i++) {
+        delete[] mMonProcessBuffer[i];
+    }
 #else   // WAIR
-    int iCnt = (mNumInChans > mNumNetRevChans) ? mNumInChans : mNumNetRevChans;
-    int oCnt = (mNumOutChans > mNumNetRevChans) ? mNumOutChans : mNumNetRevChans;
-    int aCnt = (mNumNetRevChans) ? mNumInChans : 0;
-    for (int i = 0; i < iCnt; i++) {
+    for (int i = 0; i < mInProcessBuffer.size(); i++) {
         delete[] mInProcessBuffer[i];
     }
-    for (int i = 0; i < oCnt; i++) {
+    for (int i = 0; i < mOutProcessBuffer.size(); i++) {
         delete[] mOutProcessBuffer[i];
     }
-    for (int i = 0; i < aCnt; i++) {
+    for (int i = 0; i < mMonProcessBuffer.size(); i++) {
+        delete[] mMonProcessBuffer[i];
+    }
+    for (int i = 0; i < mAPInBuffer.size(); i++) {
         delete[] mAPInBuffer[i];
     }
 #endif  // endwhere
-
     for (auto* i : qAsConst(mProcessPluginsFromNetwork)) {
         delete i;
     }
     for (auto* i : qAsConst(mProcessPluginsToNetwork)) {
         delete i;
     }
-    for (int i = 0; i < mNumInChans; i++) {
+    for (auto* i : qAsConst(mProcessPluginsToMonitor)) {
+        delete i;
+    }
+    for (int i = 0; i < mInBufCopy.size(); i++) {
         delete[] mInBufCopy[i];
     }
 }
@@ -147,11 +170,22 @@ AudioInterface::~AudioInterface()
 //*******************************************************************************
 void AudioInterface::setup(bool /*verbose*/)
 {
+    int nChansIn  = mInputChans.size();
+    int nChansOut = mOutputChans.size();
+    int nChansMon =
+        std::min(nChansIn, nChansOut);  // Note: Should be 2 when mixing stereo-to-mono
+    inputMixModeT inputMixMode = mInputMixMode;
+    if (inputMixMode == MIXTOMONO) {
+        nChansIn = 1;
+    }
+    if (inputMixMode == MONO) {
+        nChansMon = nChansOut;
+    }
     // Allocate buffer memory to read and write
     mSizeInBytesPerChannel = getSizeInBytesPerChannel();
 
-    int size_audio_input  = mSizeInBytesPerChannel * getNumInputChannels();
-    int size_audio_output = mSizeInBytesPerChannel * getNumOutputChannels();
+    int size_audio_input  = mSizeInBytesPerChannel * nChansIn;
+    int size_audio_output = mSizeInBytesPerChannel * nChansOut;
 #ifdef WAIR               // WAIR
     if (mNumNetRevChans)  // else don't change sizes
     {
@@ -167,40 +201,52 @@ void AudioInterface::setup(bool /*verbose*/)
     if (mNumNetRevChans) {
         mInProcessBuffer.resize(mNumNetRevChans);
         mOutProcessBuffer.resize(mNumNetRevChans);
-        mAPInBuffer.resize(mNumInChans);
+        mMonProcessBuffer.resize(mNumNetRevChans);
+        mAPInBuffer.resize(nChansIn);
         mNetInBuffer.resize(mNumNetRevChans);
     } else  // don't change sizes
 #endif      // endwhere
     {
-        mInProcessBuffer.resize(mNumInChans);
-        mOutProcessBuffer.resize(mNumOutChans);
+        mInProcessBuffer.resize(nChansIn);
+        mOutProcessBuffer.resize(nChansOut);
+        mMonProcessBuffer.resize(nChansMon);
     }
 
     int nframes = getBufferSizeInSamples();
 
 #ifndef WAIR  // NOT WAIR:
-    for (int i = 0; i < mNumInChans; i++) {
+    for (int i = 0; i < nChansIn; i++) {
         mInProcessBuffer[i] = new sample_t[nframes];
         // set memory to 0
         std::memset(mInProcessBuffer[i], 0, sizeof(sample_t) * nframes);
     }
-    for (int i = 0; i < mNumOutChans; i++) {
+    for (int i = 0; i < nChansOut; i++) {
         mOutProcessBuffer[i] = new sample_t[nframes];
         // set memory to 0
         std::memset(mOutProcessBuffer[i], 0, sizeof(sample_t) * nframes);
     }
+    for (int i = 0; i < nChansMon; i++) {
+        mMonProcessBuffer[i] = new sample_t[nframes];
+        // set memory to 0
+        std::memset(mMonProcessBuffer[i], 0, sizeof(sample_t) * nframes);
+    }
 #else   // WAIR
-    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : mNumInChans); i++) {
+    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : nChansIn); i++) {
         mInProcessBuffer[i] = new sample_t[nframes];
         // set memory to 0
         std::memset(mInProcessBuffer[i], 0, sizeof(sample_t) * nframes);
     }
-    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : mNumOutChans); i++) {
+    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : nChansOut); i++) {
         mOutProcessBuffer[i] = new sample_t[nframes];
         // set memory to 0
         std::memset(mOutProcessBuffer[i], 0, sizeof(sample_t) * nframes);
     }
-    for (int i = 0; i < ((mNumNetRevChans) ? mNumInChans : 0); i++) {
+    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : nChansMon); i++) {
+        mMonProcessBuffer[i] = new sample_t[nframes];
+        // set memory to 0
+        std::memset(mMonitorProcess[i], 0, sizeof(sample_t) * nframes);
+    }
+    for (int i = 0; i < ((mNumNetRevChans) ? nChansIn : 0); i++) {
         mAPInBuffer[i] = new sample_t[nframes];
         // set memory to 0
         std::memset(mAPInBuffer[i], 0, sizeof(sample_t) * nframes);
@@ -224,6 +270,17 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
                               QVarLengthArray<sample_t*>& out_buffer,
                               unsigned int n_frames)
 {
+    int nChansIn  = mInputChans.size();
+    int nChansOut = mOutputChans.size();
+    int nChansMon =
+        std::min(nChansIn, nChansOut);  // Note: Should be 2 when mixing stereo-to-mono
+    inputMixModeT inputMixMode = mInputMixMode;
+    if (inputMixMode == MIXTOMONO) {
+        nChansIn = 1;
+    }
+    if (inputMixMode == MONO) {
+        nChansMon = nChansOut;
+    }
     // Allocate the Process Callback
     //-------------------------------------------------------------------
     // 1) First, process incoming packets
@@ -271,10 +328,10 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
         }
     }
 #else   // WAIR:
-    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : mNumOutChans); i++) {
+    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : nChansOut); i++) {
         std::memset(mOutProcessBuffer[i], 0, sizeof(sample_t) * n_frames);
     }
-    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : mNumInChans); i++) {
+    for (int i = 0; i < ((mNumNetRevChans) ? mNumNetRevChans : nChansIn); i++) {
         std::memset(mInProcessBuffer[i], 0, sizeof(sample_t) * n_frames);
         if (mNumNetRevChans) {
             if (client)
@@ -298,9 +355,11 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
     // mAudioTesterP will be nullptr for hub server's JackTripWorker instances:
     bool audioTesting = (mAudioTesterP && mAudioTesterP->getEnabled());
     int nop = mProcessPluginsToNetwork.size();  // number of OUTGOING processing modules
-    if (nop > 0 || audioTesting) {              // cannot modify in_buffer, so make a copy
+    if (nop > 0 || audioTesting
+        || mProcessPluginsToMonitor.size()
+               > 0) {  // cannot modify in_buffer, so make a copy
         // in_buffer is "in" from local audio hardware via JACK
-        if (mInBufCopy.size() < mNumInChans) {  // created in constructor above
+        if (mInBufCopy.size() < nChansIn) {  // created in constructor above
             std::cerr << "*** AudioInterface.cpp: Number of Input Channels changed - "
                          "insufficient room reserved\n";
             exit(1);
@@ -310,7 +369,7 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
                       << " larger than expected max = " << MAX_AUDIO_BUFFER_SIZE << "\n";
             exit(1);
         }
-        for (int i = 0; i < mNumInChans; i++) {
+        for (int i = 0; i < nChansIn; i++) {
             std::memcpy(mInBufCopy[i], in_buffer[i], sizeof(sample_t) * n_frames);
         }
         for (int i = 0; i < nop; i++) {
@@ -320,6 +379,31 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
                 p->compute(n_frames, mInBufCopy.data(), mInBufCopy.data());
             }
         }
+
+        for (int i = 0; i < nChansMon; i++) {
+            if ((mInputChans.size() == 2 && mInputMixMode == AudioInterface::MIXTOMONO)
+                || (mInputMixMode == AudioInterface::MONO)) {
+                // if using mix-to-mono, in_buffer[0] should already contain the mixed
+                // audio, so copy it to the monitor buffer. See RtAudioInterface.cpp
+
+                // likewise if using mono, we simply copy the input to every monitor
+                // channel
+                std::memcpy(mMonProcessBuffer[i], in_buffer[0],
+                            sizeof(sample_t) * n_frames);
+            } else {
+                // otherwise, copy each channel individually
+                std::memcpy(mMonProcessBuffer[i], in_buffer[i],
+                            sizeof(sample_t) * n_frames);
+            }
+        }
+        for (int i = 0; i < mProcessPluginsToMonitor.size(); i++) {
+            ProcessPlugin* p = mProcessPluginsToMonitor[i];
+            if (p->getInited()) {
+                // note: for monitor plugins, the output is out_buffer (to the speakers)
+                p->compute(n_frames, mMonProcessBuffer.data(), out_buffer.data());
+            }
+        }
+
         if (audioTesting) {
             mAudioTesterP->writeImpulse(
                 mInBufCopy,
@@ -345,11 +429,11 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
 #define AP
 #ifndef AP
         // straight to audio out
-        for (int i = 0; i < mNumOutChans; i++) {
+        for (int i = 0; i < nChansOut; i++) {
             std::memset(out_buffer[i], 0, sizeof(sample_t) * n_frames);
         }
         for (int i = 0; i < mNumNetRevChans; i++) {
-            sample_t* mix_sample = out_buffer[i % mNumOutChans];
+            sample_t* mix_sample = out_buffer[i % nChansOut];
             sample_t* tmp_sample = mNetInBuffer[i];  // mNetInBuffer
             for (int j = 0; j < (int)n_frames; j++) {
                 mix_sample[j] += tmp_sample[j];
@@ -360,17 +444,17 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
         // output through all-pass cascade
         // AP2 is 2 channel, mixes inputs to mono, then splits to two parallel AP chains
         // AP8 is 2 channel, two parallel AP chains
-        for (int i = 0; i < mNumInChans; i++) {
+        for (int i = 0; i < nChansIn; i++) {
             std::memset(mAPInBuffer[i], 0, sizeof(sample_t) * n_frames);
         }
         for (int i = 0; i < mNumNetRevChans; i++) {
-            sample_t* mix_sample = mAPInBuffer[i % mNumOutChans];
+            sample_t* mix_sample = mAPInBuffer[i % nChansOut];
             sample_t* tmp_sample = mNetInBuffer[i];
             for (int j = 0; j < n_frames; j++) {
                 mix_sample[j] += tmp_sample[j];
             }
         }  // nib16 to apib2
-        for (int i = 0; i < mNumOutChans; i++) {
+        for (int i = 0; i < nChansOut; i++) {
             std::memset(out_buffer[i], 0, sizeof(sample_t) * n_frames);
         }
         mProcessPluginsFromNetwork[APDSP]->compute(n_frames, mAPInBuffer.data(),
@@ -379,7 +463,7 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
 
         //#define ADD_DIRECT
 #ifdef ADD_DIRECT
-        for (int i = 0; i < mNumInChans; i++) {
+        for (int i = 0; i < nChansIn; i++) {
             sample_t* mix_sample = out_buffer[i];
             sample_t* tmp_sample = in_buffer[i];
             for (int j = 0; j < n_frames; j++) {
@@ -412,20 +496,22 @@ void AudioInterface::callback(QVarLengthArray<sample_t*>& in_buffer,
 void AudioInterface::broadcastCallback(QVarLengthArray<sample_t*>& mon_buffer,
                                        unsigned int n_frames)
 {
+    int nChansOut = mOutputChans.size();
+
     /// \todo cast *mInBuffer[i] to the bit resolution
     // Output Process (from NETWORK to JACK)
     // ----------------------------------------------------------------
     // Read Audio buffer from RingBuffer (read from incoming packets)
     mJackTrip->receiveBroadcastPacket(mAudioOutputPacket);
     // Extract separate channels to send to Jack
-    for (int i = 0; i < mNumOutChans; i++) {
+    for (int i = 0; i < nChansOut; i++) {
         sample_t* tmp_sample = mon_buffer[i];  // sample buffer for channel i
         for (unsigned int j = 0; j < n_frames; j++) {
             // Change the bit resolution on each sample
             fromBitToSampleConversion(
                 // use interleaved channel layout
                 //&mOutputPacket[(i*mSizeInBytesPerChannel) + (j*mBitResolutionMode)],
-                &mAudioOutputPacket[(j * mBitResolutionMode * mNumOutChans)
+                &mAudioOutputPacket[(j * mBitResolutionMode * nChansOut)
                                     + (i * mBitResolutionMode)],
                 &tmp_sample[j], mBitResolutionMode);
         }
@@ -439,6 +525,8 @@ void AudioInterface::broadcastCallback(QVarLengthArray<sample_t*>& mon_buffer,
 void AudioInterface::computeProcessFromNetwork(QVarLengthArray<sample_t*>& out_buffer,
                                                unsigned int n_frames)
 {
+    int nChansOut = mOutputChans.size();
+
     /// \todo cast *mInBuffer[i] to the bit resolution
     // Output Process (from NETWORK to JACK)
     // ----------------------------------------------------------------
@@ -455,7 +543,7 @@ void AudioInterface::computeProcessFromNetwork(QVarLengthArray<sample_t*>& out_b
                 fromBitToSampleConversion(
                     // use interleaved channel layout
                     //&mOutputPacket[(i*mSizeInBytesPerChannel) + (j*mBitResolutionMode)],
-                    &mOutputPacket[(j * mBitResolutionMode * mNumOutChans)
+                    &mOutputPacket[(j * mBitResolutionMode * nChansOut)
                                    + (i * mBitResolutionMode)],
                     &tmp_sample[j], mBitResolutionMode);
             }
@@ -464,7 +552,7 @@ void AudioInterface::computeProcessFromNetwork(QVarLengthArray<sample_t*>& out_b
 #endif    // endwhere
 
         // Extract separate channels to send to Jack
-        for (int i = 0; i < mNumOutChans; i++) {
+        for (int i = 0; i < nChansOut; i++) {
             //--------
             // This should be faster for 32 bits
             // std::memcpy(mOutBuffer[i], &mOutputPacket[i*mSizeInBytesPerChannel],
@@ -476,7 +564,7 @@ void AudioInterface::computeProcessFromNetwork(QVarLengthArray<sample_t*>& out_b
                 fromBitToSampleConversion(
                     // use interleaved channel layout
                     //&mOutputPacket[(i*mSizeInBytesPerChannel) + (j*mBitResolutionMode)],
-                    &mAudioOutputPacket[(j * mBitResolutionMode * mNumOutChans)
+                    &mAudioOutputPacket[(j * mBitResolutionMode * nChansOut)
                                         + (i * mBitResolutionMode)],
                     &tmp_sample[j], mBitResolutionMode);
             }
@@ -487,6 +575,11 @@ void AudioInterface::computeProcessFromNetwork(QVarLengthArray<sample_t*>& out_b
 void AudioInterface::computeProcessToNetwork(QVarLengthArray<sample_t*>& in_buffer,
                                              unsigned int n_frames)
 {
+    int nChansIn               = mInputChans.size();
+    inputMixModeT inputMixMode = mInputMixMode;
+    if (inputMixMode == MIXTOMONO) {
+        nChansIn = 1;
+    }
     // Input Process (from JACK to NETWORK)
     // ----------------------------------------------------------------
     // Concatenate  all the channels from jack to form packet
@@ -495,7 +588,7 @@ void AudioInterface::computeProcessToNetwork(QVarLengthArray<sample_t*>& in_buff
     if (mNumNetRevChans)
         for (int i = 0; i < mNumNetRevChans; i++) {
             sample_t* tmp_sample =
-                in_buffer[i % mNumInChans];  // sample buffer for channel i
+                in_buffer[i % nChansIn];  // sample buffer for channel i
             sample_t* tmp_process_sample =
                 mInProcessBuffer[i];  // sample buffer from the output process
             sample_t tmp_result;
@@ -511,7 +604,7 @@ void AudioInterface::computeProcessToNetwork(QVarLengthArray<sample_t*>& in_buff
                     &tmp_result,
                     // use interleaved channel layout
                     //&mInputPacket[(i*mSizeInBytesPerChannel) + (j*mBitResolutionMode)],
-                    &mInputPacket[(j * mBitResolutionMode * mNumOutChans)
+                    &mInputPacket[(j * mBitResolutionMode * nChansOut)
                                   + (i * mBitResolutionMode)],
                     mBitResolutionMode);
             }
@@ -519,7 +612,7 @@ void AudioInterface::computeProcessToNetwork(QVarLengthArray<sample_t*>& in_buff
     else  // not wair
 #endif    // endwhere
 
-        for (int i = 0; i < mNumInChans; i++) {
+        for (int i = 0; i < nChansIn; i++) {
             //--------
             // This should be faster for 32 bits
             // std::memcpy(&mInputPacket[i*mSizeInBytesPerChannel], mInBuffer[i],
@@ -538,7 +631,7 @@ void AudioInterface::computeProcessToNetwork(QVarLengthArray<sample_t*>& in_buff
                     &tmp_result,
                     // use interleaved channel layout
                     //&mInputPacket[(i*mSizeInBytesPerChannel) + (j*mBitResolutionMode)],
-                    &mAudioInputPacket[(j * mBitResolutionMode * mNumInChans)
+                    &mAudioInputPacket[(j * mBitResolutionMode * nChansIn)
                                        + (i * mBitResolutionMode)],
                     mBitResolutionMode);
             }
@@ -650,9 +743,16 @@ void AudioInterface::appendProcessPluginToNetwork(ProcessPlugin* plugin)
     if (not plugin) {
         return;
     }
+
+    int nChansIn               = mInputChans.size();
+    inputMixModeT inputMixMode = mInputMixMode;
+    if (inputMixMode == MIXTOMONO) {
+        nChansIn = 1;
+    }
+
     int nTestChans   = (mAudioTesterP && mAudioTesterP->getEnabled()) ? 1 : 0;
-    int nPluginChans = mNumInChans - nTestChans;
-    assert(nTestChans == 0 || (mAudioTesterP->getSendChannel() == mNumInChans - 1));
+    int nPluginChans = nChansIn - nTestChans;
+    assert(nTestChans == 0 || (mAudioTesterP->getSendChannel() == nChansIn - 1));
     if (plugin->getNumInputs() < nPluginChans) {
         std::cerr
             << "*** AudioInterface.cpp: appendProcessPluginToNetwork: ProcessPlugin "
@@ -669,9 +769,12 @@ void AudioInterface::appendProcessPluginFromNetwork(ProcessPlugin* plugin)
     if (not plugin) {
         return;
     }
+
+    int nChansOut = mOutputChans.size();
+
     int nTestChans   = (mAudioTesterP && mAudioTesterP->getEnabled()) ? 1 : 0;
-    int nPluginChans = mNumOutChans - nTestChans;
-    assert(nTestChans == 0 || (mAudioTesterP->getSendChannel() == mNumOutChans - 1));
+    int nPluginChans = nChansOut - nTestChans;
+    assert(nTestChans == 0 || (mAudioTesterP->getSendChannel() == nChansOut - 1));
     if (plugin->getNumOutputs() > nPluginChans) {
         std::cerr
             << "*** AudioInterface.cpp: appendProcessPluginFromNetwork: ProcessPlugin "
@@ -683,9 +786,60 @@ void AudioInterface::appendProcessPluginFromNetwork(ProcessPlugin* plugin)
     mProcessPluginsFromNetwork.append(plugin);
 }
 
+void AudioInterface::appendProcessPluginToMonitor(ProcessPlugin* plugin)
+{
+    if (not plugin) {
+        return;
+    }
+    int nChansIn  = mInputChans.size();
+    int nChansOut = mOutputChans.size();
+    int nChansMon =
+        std::min(nChansIn, nChansOut);  // Note: Should be 2 when mixing stereo-to-mono
+    inputMixModeT inputMixMode = mInputMixMode;
+    if (inputMixMode == MIXTOMONO) {
+        nChansIn = 1;
+    }
+    if (inputMixMode == MONO) {
+        nChansMon = nChansOut;
+    }
+    if (plugin->getNumInputs() > nChansMon) {
+        std::cerr
+            << "*** AudioInterface.cpp: appendProcessPluginToMonitor: ProcessPlugin "
+            << typeid(plugin).name() << " REJECTED due to having "
+            << plugin->getNumInputs()
+            << " inputs, while the monitor audio input requires " << nChansMon
+            << " outputs\n";
+        return;
+    }
+
+    if (plugin->getNumOutputs() > nChansMon) {
+        std::cerr
+            << "*** AudioInterface.cpp: appendProcessPluginToMonitor: ProcessPlugin "
+            << typeid(plugin).name() << " REJECTED due to having "
+            << plugin->getNumOutputs()
+            << " inputs, while the monitor audio output requires " << nChansMon
+            << " outputs\n";
+        return;
+    }
+
+    mProcessPluginsToMonitor.append(plugin);
+}
+
 void AudioInterface::initPlugins(bool verbose)
 {
-    int nPlugins = mProcessPluginsFromNetwork.size() + mProcessPluginsToNetwork.size();
+    int nChansIn  = mInputChans.size();
+    int nChansOut = mOutputChans.size();
+    int nChansMon =
+        std::min(nChansIn, nChansOut);  // Note: Should be 2 when mixing stereo-to-mono
+    inputMixModeT inputMixMode = mInputMixMode;
+    if (inputMixMode == MIXTOMONO) {
+        nChansIn = 1;
+    }
+    if (inputMixMode == MONO) {
+        nChansMon = nChansOut;
+    }
+    int nPlugins = mProcessPluginsFromNetwork.size() + mProcessPluginsToNetwork.size()
+                   + mProcessPluginsToMonitor.size();
     if (nPlugins > 0) {
         if (verbose) {
             std::cout << "Initializing Faust plugins (have " << nPlugins
@@ -694,13 +848,18 @@ void AudioInterface::initPlugins(bool verbose)
 
         for (ProcessPlugin* plugin : qAsConst(mProcessPluginsFromNetwork)) {
             plugin->setOutgoingToNetwork(false);
-            plugin->updateNumChannels(mNumInChans, mNumOutChans);
-            plugin->init(mSampleRate);
+            plugin->updateNumChannels(nChansIn, nChansOut);
+            plugin->init(mSampleRate, mBufferSizeInSamples);
         }
         for (ProcessPlugin* plugin : qAsConst(mProcessPluginsToNetwork)) {
             plugin->setOutgoingToNetwork(true);
-            plugin->updateNumChannels(mNumInChans, mNumOutChans);
-            plugin->init(mSampleRate);
+            plugin->updateNumChannels(nChansIn, nChansOut);
+            plugin->init(mSampleRate, mBufferSizeInSamples);
+        }
+        for (ProcessPlugin* plugin : qAsConst(mProcessPluginsToMonitor)) {
+            plugin->setOutgoingToNetwork(false);
+            plugin->updateNumChannels(nChansMon, nChansMon);
+            plugin->init(mSampleRate, mBufferSizeInSamples);
         }
     }
 }
@@ -776,9 +935,9 @@ void AudioInterface::setDevicesWarningMsg(warningMessageT msg)
     switch (msg) {
     case DEVICE_WARN_LATENCY:
         mWarningMsg =
-            "The selected devices don't support low latency. You can use them, but you "
-            "will experience audio delay. Make sure you have up to date drivers from the "
-            "manufacturer!";
+            "The selected Input and Output devices are Non-ASIO and may cause high "
+            "latency or audio delay. Installing ASIO drivers and using ASIO Input and "
+            "Output devices will lower audio delays for a 'same room experience'.";
 #ifdef _WIN32
         mWarningHelpUrl = "https://help.jacktrip.org/hc/en-us/articles/4409919243155";
 #else
@@ -821,6 +980,13 @@ void AudioInterface::setDevicesErrorMsg(errorMessageT msg)
         mErrorMsg     = "JackTrip couldn't find any audio devices!";
         mErrorHelpUrl = "";
         break;
+#ifdef _WIN32
+    case DEVICE_ERR_SAME_ASIO:
+        mErrorMsg =
+            "When using ASIO, please select the same device for your input and output.";
+        mErrorHelpUrl = "https://help.jacktrip.org/hc/en-us/articles/4409919243155";
+        break;
+#endif
     default:
         mErrorMsg     = "";
         mErrorHelpUrl = "";
@@ -851,4 +1017,4 @@ std::string AudioInterface::getDevicesWarningHelpUrl()
 std::string AudioInterface::getDevicesErrorHelpUrl()
 {
     return mErrorHelpUrl;
-}
\ No newline at end of file
+}
