@@ -31,7 +31,7 @@
 
 /**
  * \file virtualstudio.h
- * \author Aaron Wyatt
+ * \author Matt Horton, based on code by Aaron Wyatt
  * \date March 2022
  */
 
@@ -41,16 +41,21 @@
 #include <QEventLoop>
 #include <QList>
 #include <QMutex>
+#include <QNetworkAccessManager>
 #include <QScopedPointer>
 #include <QSharedPointer>
 #include <QTimer>
 #include <QVector>
 #include <QtNetworkAuth>
 
+#include "../Analyzer.h"
 #include "../JackTrip.h"
 #include "../Meter.h"
+#include "../Monitor.h"
 #include "../Volume.h"
+#include "vsApi.h"
 #include "vsAudioInterface.h"
+#include "vsAuth.h"
 #include "vsConstants.h"
 #include "vsDevice.h"
 #include "vsQuickView.h"
@@ -78,27 +83,44 @@ class VirtualStudio : public QObject
     Q_PROPERTY(bool selectableBackend READ selectableBackend CONSTANT)
     Q_PROPERTY(QString audioBackend READ audioBackend WRITE setAudioBackend NOTIFY
                    audioBackendChanged)
-    Q_PROPERTY(
-        int inputDevice READ inputDevice WRITE setInputDevice NOTIFY inputDeviceChanged)
-    Q_PROPERTY(int outputDevice READ outputDevice WRITE setOutputDevice NOTIFY
+    Q_PROPERTY(QString inputDevice READ inputDevice WRITE setInputDevice NOTIFY
+                   inputDeviceChanged)
+    Q_PROPERTY(QString outputDevice READ outputDevice WRITE setOutputDevice NOTIFY
                    outputDeviceChanged)
     Q_PROPERTY(int previousInput READ previousInput WRITE setPreviousInput NOTIFY
                    previousInputChanged)
     Q_PROPERTY(int previousOutput READ previousOutput WRITE setPreviousOutput NOTIFY
                    previousOutputChanged)
-
+#ifdef RT_AUDIO
+    Q_PROPERTY(int baseInputChannel READ baseInputChannel WRITE setBaseInputChannel NOTIFY
+                   baseInputChannelChanged)
+    Q_PROPERTY(int numInputChannels READ numInputChannels WRITE setNumInputChannels NOTIFY
+                   numInputChannelsChanged)
+    Q_PROPERTY(int inputMixMode READ inputMixMode WRITE setInputMixMode NOTIFY
+                   inputMixModeChanged)
+    Q_PROPERTY(int baseOutputChannel READ baseOutputChannel WRITE setBaseOutputChannel
+                   NOTIFY baseOutputChannelChanged)
+    Q_PROPERTY(int numOutputChannels READ numOutputChannels WRITE setNumOutputChannels
+                   NOTIFY numOutputChannelsChanged)
+#endif
     Q_PROPERTY(QString devicesWarning READ devicesWarning NOTIFY devicesWarningChanged)
     Q_PROPERTY(QString devicesError READ devicesError NOTIFY devicesErrorChanged)
     Q_PROPERTY(QString devicesWarningHelpUrl READ devicesWarningHelpUrl NOTIFY
                    devicesWarningHelpUrlChanged)
     Q_PROPERTY(QString devicesErrorHelpUrl READ devicesErrorHelpUrl NOTIFY
                    devicesErrorHelpUrlChanged)
+    Q_PROPERTY(
+        QString connectedErrorMsg READ connectedErrorMsg NOTIFY connectedErrorMsgChanged)
 
     Q_PROPERTY(
         int bufferSize READ bufferSize WRITE setBufferSize NOTIFY bufferSizeChanged)
     Q_PROPERTY(int bufferStrategy READ bufferStrategy WRITE setBufferStrategy NOTIFY
                    bufferStrategyChanged)
+    Q_PROPERTY(bool feedbackDetectionEnabled READ feedbackDetectionEnabled WRITE
+                   setFeedbackDetectionEnabled NOTIFY feedbackDetectionEnabledChanged)
     Q_PROPERTY(int currentStudio READ currentStudio NOTIFY currentStudioChanged)
+    Q_PROPERTY(QUrl studioToJoin READ studioToJoin WRITE setStudioToJoin NOTIFY
+                   studioToJoinChanged)
     Q_PROPERTY(QJsonObject regions READ regions NOTIFY regionsChanged)
     Q_PROPERTY(QJsonObject userMetadata READ userMetadata NOTIFY userMetadataChanged)
     Q_PROPERTY(bool showInactive READ showInactive WRITE setShowInactive NOTIFY
@@ -109,6 +131,7 @@ class VirtualStudio : public QObject
                    NOTIFY showCreateStudioChanged)
     Q_PROPERTY(QString connectionState READ connectionState NOTIFY connectionStateChanged)
     Q_PROPERTY(QJsonObject networkStats READ networkStats NOTIFY networkStatsChanged)
+    Q_PROPERTY(bool networkOutage READ networkOutage NOTIFY updatedNetworkOutage)
 
     Q_PROPERTY(QString updateChannel READ updateChannel WRITE setUpdateChannel NOTIFY
                    updateChannelChanged)
@@ -127,8 +150,26 @@ class VirtualStudio : public QObject
         float inputVolume READ inputVolume WRITE setInputVolume NOTIFY updatedInputVolume)
     Q_PROPERTY(float outputVolume READ outputVolume WRITE setOutputVolume NOTIFY
                    updatedOutputVolume)
+    Q_PROPERTY(float monitorVolume READ monitorVolume WRITE setMonitorVolume NOTIFY
+                   updatedMonitorVolume)
     Q_PROPERTY(
         bool inputMuted READ inputMuted WRITE setInputMuted NOTIFY updatedInputMuted)
+    Q_PROPERTY(QVector<float> outputMeterLevels READ outputMeterLevels NOTIFY
+                   updatedOutputMeterLevels)
+    Q_PROPERTY(QVector<float> inputMeterLevels READ inputMeterLevels NOTIFY
+                   updatedInputMeterLevels)
+    Q_PROPERTY(
+        QJsonArray inputComboModel READ inputComboModel NOTIFY inputComboModelChanged)
+    Q_PROPERTY(
+        QJsonArray outputComboModel READ outputComboModel NOTIFY outputComboModelChanged)
+    Q_PROPERTY(QJsonArray inputChannelsComboModel READ inputChannelsComboModel NOTIFY
+                   inputChannelsComboModelChanged)
+    Q_PROPERTY(QJsonArray outputChannelsComboModel READ outputChannelsComboModel NOTIFY
+                   outputChannelsComboModelChanged)
+    Q_PROPERTY(QJsonArray inputMixModeComboModel READ inputMixModeComboModel NOTIFY
+                   inputMixModeComboModelChanged)
+    Q_PROPERTY(bool inputClipped READ inputClipped NOTIFY updatedInputClipped)
+    Q_PROPERTY(bool outputClipped READ outputClipped NOTIFY updatedOutputClipped)
     Q_PROPERTY(bool audioActivated READ audioActivated WRITE setAudioActivated NOTIFY
                    audioActivatedChanged)
     Q_PROPERTY(
@@ -137,6 +178,7 @@ class VirtualStudio : public QObject
     Q_PROPERTY(QString windowState READ windowState WRITE setWindowState NOTIFY
                    windowStateUpdated)
     Q_PROPERTY(QString apiHost READ apiHost WRITE setApiHost NOTIFY apiHostChanged)
+    Q_PROPERTY(bool vsFtux READ vsFtux CONSTANT)
 
    public:
     explicit VirtualStudio(bool firstRun = false, QObject* parent = nullptr);
@@ -145,6 +187,7 @@ class VirtualStudio : public QObject
     void setStandardWindow(QSharedPointer<QJackTrip> window);
     void show();
     void raiseToTop();
+    bool vsModeActive();
 
     bool showFirstRun();
     void setShowFirstRun(bool show);
@@ -154,10 +197,24 @@ class VirtualStudio : public QObject
     bool selectableBackend();
     QString audioBackend();
     void setAudioBackend(const QString& backend);
-    int inputDevice();
-    void setInputDevice(int device);
-    int outputDevice();
-    void setOutputDevice(int device);
+    QString inputDevice();
+    void setInputDevice(const QString& device);
+#ifdef RT_AUDIO
+    int baseInputChannel();
+    void setBaseInputChannel(int baseChannel);
+    int numInputChannels();
+    void setNumInputChannels(int numChannels);
+    void setInputMixMode(int mode);
+    int inputMixMode();
+#endif
+    QString outputDevice();
+    void setOutputDevice(const QString& device);
+#ifdef RT_AUDIO
+    int baseOutputChannel();
+    void setBaseOutputChannel(int baseChannel);
+    int numOutputChannels();
+    void setNumOutputChannels(int numChannels);
+#endif
     int previousInput();
     void setPreviousInput(int device);
     int previousOutput();
@@ -166,17 +223,26 @@ class VirtualStudio : public QObject
     QString devicesError();
     QString devicesWarningHelpUrl();
     QString devicesErrorHelpUrl();
+    QString connectedErrorMsg();
+    void setConnectedErrorMsg(const QString& msg);
     int bufferSize();
     void setBufferSize(int index);
     int bufferStrategy();
     void setBufferStrategy(int index);
+    bool feedbackDetectionEnabled();
+    void setFeedbackDetectionEnabled(bool enabled);
     int currentStudio();
     QJsonObject regions();
     QJsonObject userMetadata();
     QString connectionState();
     QJsonObject networkStats();
-    QVector<float> inputMeterLevels();
-    QVector<float> outputMeterLevels();
+    const QVector<float>& inputMeterLevels() const;
+    const QVector<float>& outputMeterLevels() const;
+    const QJsonArray& inputComboModel() const;
+    const QJsonArray& outputComboModel() const;
+    const QJsonArray& inputChannelsComboModel() const;
+    const QJsonArray& outputChannelsComboModel() const;
+    const QJsonArray& inputMixModeComboModel() const;
     QString updateChannel();
     void setUpdateChannel(const QString& channel);
     bool showInactive();
@@ -203,15 +269,21 @@ class VirtualStudio : public QObject
     QString failedMessage();
     float inputVolume();
     float outputVolume();
+    float monitorVolume();
     bool inputMuted();
     bool outputMuted();
+    bool monitorMuted();
     Q_INVOKABLE void restartAudio();
     bool audioActivated();
     bool audioReady();
+    bool inputClipped();
+    bool outputClipped();
+    bool networkOutage();
     bool backendAvailable();
     QString windowState();
     QString apiHost();
     void setApiHost(QString host);
+    bool vsFtux();
 
    public slots:
     void toStandard();
@@ -220,11 +292,16 @@ class VirtualStudio : public QObject
     void logout();
     void refreshStudios(int index, bool signalRefresh = false);
     void refreshDevices();
+    void refreshRtAudioDevices();
+    void validateDevicesState();
+    void validateInputDevicesState();
+    void validateOutputDevicesState();
     void playOutputAudio();
     void revertSettings();
     void applySettings();
     void connectToStudio(int studioIndex);
     void completeConnection();
+    void triggerReconnect();
     void disconnect();
     void manageStudio(int studioIndex, bool start = false);
     void launchVideo(int studioIndex);
@@ -232,14 +309,18 @@ class VirtualStudio : public QObject
     void editProfile();
     void showAbout();
     void openLink(const QString& url);
-    void updatedInputVuMeasurements(const QVector<float>& valuesInDecibels);
-    void updatedOutputVuMeasurements(const QVector<float>& valuesInDecibels);
+    void updatedInputVuMeasurements(const float* valuesInDecibels, int numChannels);
+    void updatedOutputVuMeasurements(const float* valuesInDecibels, int numChannels);
+    void udpWaitingTooLong();
     void setInputVolume(float multiplier);
     void setOutputVolume(float multiplier);
+    void setMonitorVolume(float multiplier);
     void setInputMuted(bool muted);
     void setOutputMuted(bool muted);
+    void setMonitorMuted(bool muted);
     void setAudioActivated(bool activated);
     void setAudioReady(bool ready);
+    void detectedFeedbackLoop();
     void setWindowState(QString state);
     void exit();
 
@@ -255,18 +336,23 @@ class VirtualStudio : public QObject
     void logoSectionChanged();
     void audioBackendChanged(bool useRtAudio, bool shouldRestart = true);
     void inputDeviceChanged(QString device, bool shouldRestart = true);
+    void baseInputChannelChanged(int baseChannel, bool shouldRestart = true);
+    void numInputChannelsChanged(int numChannels, bool shouldRestart = true);
+    void inputMixModeChanged(int mode, bool shouldRestart = true);
     void outputDeviceChanged(QString device, bool shouldRestart = true);
-    void inputDeviceSelected(QString device, bool shouldRestart = true);
-    void outputDeviceSelected(QString device, bool shouldRestart = true);
+    void baseOutputChannelChanged(int baseChannel, bool shouldRestart = true);
+    void numOutputChannelsChanged(int numChannels, bool shouldRestart = true);
     void previousInputChanged();
     void previousOutputChanged();
     void devicesWarningChanged();
     void devicesErrorChanged();
     void devicesWarningHelpUrlChanged();
     void devicesErrorHelpUrlChanged();
+    void connectedErrorMsgChanged();
     void triggerPlayOutputAudio();
     void bufferSizeChanged();
     void bufferStrategyChanged();
+    void feedbackDetectionEnabledChanged();
     void currentStudioChanged();
     void regionsChanged();
     void userMetadataChanged();
@@ -282,21 +368,34 @@ class VirtualStudio : public QObject
     void newScale();
     void darkModeChanged();
     void testModeChanged();
+    void feedbackDetected();
     void signalExit();
     void periodicRefresh();
     void failedMessageChanged();
     void studioToJoinChanged();
     void updatedInputVolume(float multiplier);
     void updatedOutputVolume(float multiplier);
+    void updatedMonitorVolume(float multiplier);
     void updatedInputMuted(bool muted);
     void updatedOutputMuted(bool muted);
+    void updatedMonitorMuted(bool muted);
+    void updatedInputMeterLevels(const QVector<float>& levels);
+    void updatedOutputMeterLevels(const QVector<float>& levels);
+    void inputComboModelChanged();
+    void outputComboModelChanged();
+    void inputChannelsComboModelChanged();
+    void outputChannelsComboModelChanged();
+    void inputMixModeComboModelChanged();
+    void updatedInputClipped(bool clip);
+    void updatedOutputClipped(bool clip);
+    void updatedNetworkOutage(bool outage);
     void audioActivatedChanged();
     void audioReadyChanged();
     void windowStateUpdated();
     void apiHostChanged();
 
    private slots:
-    void slotAuthSucceded();
+    void slotAuthSucceeded();
     void slotAuthFailed();
     void processFinished();
     void processError(const QString& errorMessage);
@@ -313,8 +412,6 @@ class VirtualStudio : public QObject
     void updatedDevicesWarningHelpUrl(const QString& url);
 
    private:
-    void setupAuthenticator();
-
     void sendHeartbeat();
     void getServerList(bool firstLoad = false, bool signalRefresh = false,
                        int index = -1);
@@ -324,20 +421,25 @@ class VirtualStudio : public QObject
     void getUserMetadata();
     void stopStudio();
     void toggleAudio();
+    void resetMeters();
     void stopAudio();
     bool readyToJoin();
 #ifdef RT_AUDIO
-    QVariant formatDeviceList(const QStringList& devices, const QStringList& categories);
+    QJsonArray formatDeviceList(const QStringList& devices, const QStringList& categories,
+                                const QList<int>& channels);
 #endif
 
     bool m_showFirstRun = false;
     bool m_checkSsl     = true;
+    bool m_vsModeActive = false;
     QString m_updateChannel;
     QString m_refreshToken;
     QString m_userId;
     VsQuickView m_view;
     QSharedPointer<QJackTrip> m_standardWindow;
-    QScopedPointer<QOAuth2AuthorizationCodeFlow> m_authenticator;
+    QScopedPointer<VsAuth> m_auth;
+    QScopedPointer<VsApi> m_api;
+    QScopedPointer<QNetworkAccessManager> m_networkAccessManager;
 
     QList<QObject*> m_servers;
     QStringList m_subscribedServers;
@@ -376,51 +478,85 @@ class VirtualStudio : public QObject
     float m_fontScale        = 1;
     float m_uiScale;
     float m_previousUiScale;
-    int m_bufferStrategy    = 0;
-    QString m_apiHost       = PROD_API_HOST;
-    bool m_darkMode         = false;
-    bool m_testMode         = false;
-    QString m_failedMessage = "";
+    int m_bufferStrategy            = 0;
+    bool m_feedbackDetectionEnabled = true;
+    QString m_apiHost               = PROD_API_HOST;
+    bool m_darkMode                 = false;
+    bool m_testMode                 = false;
+    QString m_failedMessage         = "";
     QUrl m_studioToJoin;
     bool m_authenticated  = false;
     bool m_audioActivated = false;
     bool m_audioReady     = false;
+    bool m_inputClipped   = false;
+    bool m_outputClipped  = false;
+    bool m_networkOutage  = false;
 
+    Analyzer* m_inputAnalyzerPlugin;
+    Analyzer* m_outputAnalyzerPlugin;
+    QVector<float> m_inputMeterLevels;
+    QVector<float> m_outputMeterLevels;
+    QJsonArray m_inputComboModel;
+    QJsonArray m_outputComboModel;
+    QJsonArray m_inputChannelsComboModel;
+    QJsonArray m_outputChannelsComboModel;
+    QJsonArray m_inputMixModeComboModel;
     Meter* m_inputMeter;
     Meter* m_outputMeter;
     Meter* m_inputTestMeter;
     Volume* m_inputVolumePlugin;
     Volume* m_outputVolumePlugin;
+    Monitor* m_monitor;
     QTimer m_inputClipTimer;
     QTimer m_outputClipTimer;
+    QTimer m_networkOutageTimer;
 
     QString m_devicesWarningMsg     = QStringLiteral("");
     QString m_devicesErrorMsg       = QStringLiteral("");
     QString m_devicesWarningHelpUrl = QStringLiteral("");
     QString m_devicesErrorHelpUrl   = QStringLiteral("");
     QString m_windowState           = QStringLiteral("login");
+    QString m_connectedErrorMsg     = QStringLiteral("");
 
     float m_meterMax = 0.0;
     float m_meterMin = -64.0;
 
     float m_inMultiplier  = 1.0;
     float m_outMultiplier = 1.0;
+    float m_monMultiplier = 1.0;
     bool m_inMuted        = false;
     bool m_outMuted       = false;
+    bool m_monMuted       = false;
 
     QSharedPointer<VsAudioInterface> m_vsAudioInterface;
 
+#ifdef VS_FTUX
+    bool m_vsFtux = true;
+#else
+    bool m_vsFtux = false;
+#endif
+
 #ifdef RT_AUDIO
     QStringList m_inputDeviceList;
     QStringList m_outputDeviceList;
     QStringList m_inputDeviceCategories;
     QStringList m_outputDeviceCategories;
+    QList<int> m_inputDeviceChannels;
+    QList<int> m_outputDeviceChannels;
     QString m_inputDevice;
     QString m_outputDevice;
     quint16 m_bufferSize;
     QString m_previousInput;
     QString m_previousOutput;
     quint16 m_previousBuffer;
+
+    int m_baseInputChannel;
+    int m_numInputChannels;
+    int m_inputMixMode;
+
+    int m_baseOutputChannel;
+    int m_numOutputChannels;
+
     bool m_previousUseRtAudio = false;
     inline void delay(int millisecondsWait)
     {
@@ -433,8 +569,10 @@ class VirtualStudio : public QObject
 #endif
     QStringList m_bufferOptions         = {"16", "32", "64", "128", "256", "512", "1024"};
     QStringList m_bufferStrategyOptions = {"Minimal Latency", "Stable Latency",
-                                           "Loss Concealment"};
+                                           "Loss Concealment (3)",
+                                           "Loss Concealment (4)"};
     QStringList m_updateChannelOptions  = {"Stable", "Edge"};
+    QStringList m_feedbackDetectionOptions = {"Enabled", "Disabled"};
 
 #ifdef __APPLE__
     NoNap m_noNap;
