@@ -161,8 +161,21 @@ void VsAudioInterface::setupJackAudio()
         if (gVerboseFlag)
             std::cout << "  JackTrip:setupAudio before new JackAudioInterface"
                       << std::endl;
-        m_audioInterface.reset(new JackAudioInterface(
-            m_numAudioChansIn, m_numAudioChansOut, m_audioBitResolution));
+
+        QVarLengthArray<int> inputChans;
+        QVarLengthArray<int> outputChans;
+        inputChans.resize(m_numAudioChansIn);
+        outputChans.resize(m_numAudioChansOut);
+
+        for (int i = 0; i < m_numAudioChansIn; i++) {
+            inputChans[i] = 1 + i;
+        }
+        for (int i = 0; i < m_numAudioChansOut; i++) {
+            outputChans[i] = 1 + i;
+        }
+
+        m_audioInterface.reset(
+            new JackAudioInterface(inputChans, outputChans, m_audioBitResolution));
 
         m_audioInterface->setClientName(QStringLiteral("JackTrip"));
 
@@ -211,20 +224,31 @@ void VsAudioInterface::setupRtAudio()
 #ifdef RT_AUDIO
     if constexpr (isBackendAvailable<AudioInterfaceMode::ALL>()
                   || isBackendAvailable<AudioInterfaceMode::RTAUDIO>()) {
-        m_audioInterface.reset(new RtAudioInterface(m_numAudioChansIn, m_numAudioChansOut,
-                                                    m_audioBitResolution));
+        QVarLengthArray<int> inputChans;
+        QVarLengthArray<int> outputChans;
+        inputChans.resize(m_numAudioChansIn);
+        outputChans.resize(m_numAudioChansOut);
+
+        for (int i = 0; i < m_numAudioChansIn; i++) {
+            inputChans[i] = m_baseInputChannel + i;
+        }
+        for (int i = 0; i < m_numAudioChansOut; i++) {
+            outputChans[i] = m_baseOutputChannel + i;
+        }
+
+        m_audioInterface.reset(new RtAudioInterface(
+            inputChans, outputChans,
+            static_cast<AudioInterface::inputMixModeT>(m_inputMixMode),
+            m_audioBitResolution));
         m_audioInterface->setSampleRate(m_sampleRate);
         m_audioInterface->setDeviceID(m_deviceID);
         m_audioInterface->setInputDevice(m_inputDeviceName);
         m_audioInterface->setOutputDevice(m_outputDeviceName);
         m_audioInterface->setBufferSizeInSamples(m_audioBufferSize);
+        static_cast<RtAudioInterface*>(m_audioInterface.get())->setDevices(m_devices);
 
+        // Note: setup might change the number of channels and/or buffer size
         m_audioInterface->setup(true);
-        // Setup might have reduced number of channels
-        m_numAudioChansIn  = m_audioInterface->getNumInputChannels();
-        m_numAudioChansOut = m_audioInterface->getNumOutputChannels();
-        // Setup might have changed buffer size
-        m_audioBufferSize = m_audioInterface->getBufferSizeInSamples();
 
         std::string devicesWarningMsg     = m_audioInterface->getDevicesWarningMsg();
         std::string devicesErrorMsg       = m_audioInterface->getDevicesErrorMsg();
@@ -270,9 +294,7 @@ void VsAudioInterface::closeAudio()
             emit errorToProcess(QString::fromUtf8(e.what()));
         }
         m_audioInterface.clear();
-        m_numAudioChansIn  = gDefaultNumInChannels;
-        m_numAudioChansOut = gDefaultNumOutChannels;
-        m_deviceID         = gDefaultDeviceID;
+        m_deviceID = gDefaultDeviceID;
     }
 }
 
@@ -286,9 +308,14 @@ void VsAudioInterface::replaceProcess()
     }
 }
 
-void VsAudioInterface::processMeterMeasurements(QVector<float> values)
+void VsAudioInterface::processInputMeterMeasurements(float* values, int numChannels)
 {
-    emit newVolumeMeterMeasurements(values);
+    emit newInputMeterMeasurements(values, numChannels);
+}
+
+void VsAudioInterface::processOutputMeterMeasurements(float* values, int numChannels)
+{
+    emit newOutputMeterMeasurements(values, numChannels);
 }
 
 void VsAudioInterface::addInputPlugin(ProcessPlugin* plugin)
@@ -301,15 +328,15 @@ void VsAudioInterface::addOutputPlugin(ProcessPlugin* plugin)
     m_audioInterface->appendProcessPluginFromNetwork(plugin);
 }
 
+void VsAudioInterface::addMonitorPlugin(ProcessPlugin* plugin)
+{
+    m_audioInterface->appendProcessPluginToMonitor(plugin);
+}
+
 void VsAudioInterface::setInputDevice(QString deviceName, bool shouldRestart)
 {
     m_inputDeviceName = deviceName.toStdString();
-    if (m_inputDeviceName == "(default)") {
-        m_inputDeviceName = "";
-    }
-
     if (!m_audioInterface.isNull()) {
-        m_audioInterface->setInputDevice(m_inputDeviceName);
         if (m_audioActive && shouldRestart) {
             emit settingsUpdated();
         }
@@ -319,18 +346,229 @@ void VsAudioInterface::setInputDevice(QString deviceName, bool shouldRestart)
 void VsAudioInterface::setOutputDevice(QString deviceName, bool shouldRestart)
 {
     m_outputDeviceName = deviceName.toStdString();
-    if (m_outputDeviceName == "(default)") {
-        m_outputDeviceName = "";
-    }
-
     if (!m_audioInterface.isNull()) {
-        m_audioInterface->setOutputDevice(m_outputDeviceName);
         if (m_audioActive && shouldRestart) {
             emit settingsUpdated();
         }
     }
 }
 
+#ifdef RT_AUDIO
+void VsAudioInterface::setBaseInputChannel(int baseChannel, bool shouldRestart)
+{
+    if (m_audioInterfaceMode != VsAudioInterface::RTAUDIO) {
+        return;
+    }
+    m_baseInputChannel = baseChannel;
+    if (!m_audioInterface.isNull()) {
+        if (m_audioActive && shouldRestart) {
+            emit settingsUpdated();
+        }
+    }
+    return;
+}
+
+void VsAudioInterface::setNumInputChannels(int numChannels, bool shouldRestart)
+{
+    if (m_audioInterfaceMode != VsAudioInterface::RTAUDIO) {
+        return;
+    }
+    m_numAudioChansIn = numChannels;
+    if (!m_audioInterface.isNull()) {
+        if (m_audioActive && shouldRestart) {
+            emit settingsUpdated();
+        }
+    }
+}
+
+void VsAudioInterface::setInputMixMode(const int mode, bool shouldRestart)
+{
+    if (m_audioInterfaceMode != VsAudioInterface::RTAUDIO) {
+        return;
+    }
+    m_inputMixMode = mode;
+    if (!m_audioInterface.isNull()) {
+        if (m_audioActive && shouldRestart) {
+            emit settingsUpdated();
+        }
+    }
+    return;
+}
+
+void VsAudioInterface::setBaseOutputChannel(int baseChannel, bool shouldRestart)
+{
+    if (m_audioInterfaceMode != VsAudioInterface::RTAUDIO) {
+        return;
+    }
+    m_baseOutputChannel = baseChannel;
+    if (!m_audioInterface.isNull()) {
+        if (m_audioActive && shouldRestart) {
+            emit settingsUpdated();
+        }
+    }
+    return;
+}
+
+void VsAudioInterface::setNumOutputChannels(int numChannels, bool shouldRestart)
+{
+    if (m_audioInterfaceMode != VsAudioInterface::RTAUDIO) {
+        return;
+    }
+    m_numAudioChansOut = numChannels;
+    if (!m_audioInterface.isNull()) {
+        if (m_audioActive && shouldRestart) {
+            emit settingsUpdated();
+        }
+    }
+}
+
+void VsAudioInterface::refreshRtAudioDevices()
+{
+    RtAudioInterface::scanDevices(m_devices);
+}
+
+void VsAudioInterface::getDeviceList(QStringList* list, QStringList* categories,
+                                     QList<int>* channels, bool isInput)
+{
+    RtAudio baseRtAudio;
+    RtAudio::Api baseRtAudioApi = baseRtAudio.getCurrentApi();
+    if (categories != NULL) {
+        categories->clear();
+    }
+    if (channels != NULL) {
+        channels->clear();
+    }
+    list->clear();
+
+    // do not include blacklisted audio interfaces
+    // these are known to be unstable and cause JackTrip to crash
+    QVector<QString> blacklisted_devices = {
+#ifdef _WIN32
+        // Realtek ASIO: seems to crash any computer that tries to use it
+        QString::fromUtf8("Realtek ASIO"),
+#endif
+        // JackRouter: crashes if not running; use Jack backend instead
+        QString::fromUtf8("JackRouter"),
+    };
+
+    // Explicitly add default device
+    QString defaultDeviceName = "";
+    uint32_t defaultDeviceIdx;
+    RtAudio::DeviceInfo defaultDeviceInfo;
+    if (isInput) {
+        defaultDeviceIdx = baseRtAudio.getDefaultInputDevice();
+    } else {
+        defaultDeviceIdx = baseRtAudio.getDefaultOutputDevice();
+    }
+
+    if (defaultDeviceIdx != 0) {
+        defaultDeviceInfo = baseRtAudio.getDeviceInfo(defaultDeviceIdx);
+        defaultDeviceName = QString::fromStdString(defaultDeviceInfo.name);
+    }
+
+    if (blacklisted_devices.contains(defaultDeviceName)) {
+        std::cout << "RTAudio: blacklisted default " << (isInput ? "input" : "output")
+                  << " device: " << defaultDeviceName.toStdString() << std::endl;
+    } else if (defaultDeviceName != "") {
+        list->append(defaultDeviceName);
+        if (categories != NULL) {
+#ifdef _WIN32
+            switch (baseRtAudioApi) {
+            case RtAudio::WINDOWS_ASIO:
+                categories->append(QStringLiteral("Low-Latency (ASIO)"));
+                break;
+            case RtAudio::WINDOWS_WASAPI:
+                categories->append(QStringLiteral("High-Latency (Non-ASIO)"));
+                break;
+            case RtAudio::WINDOWS_DS:
+                categories->append(QStringLiteral("High-Latency (Non-ASIO)"));
+                break;
+            default:
+                categories->append(QStringLiteral(""));
+                break;
+            }
+#else
+            categories->append(QStringLiteral(""));
+#endif
+        }
+        if (channels != NULL) {
+            if (isInput) {
+                channels->append(defaultDeviceInfo.inputChannels);
+            } else {
+                channels->append(defaultDeviceInfo.outputChannels);
+            }
+        }
+    }
+
+    for (int n = 0; n < m_devices.size(); ++n) {
+#ifdef _WIN32
+        if (m_devices[n].api == RtAudio::UNIX_JACK) {
+            continue;
+        }
+#endif
+        // Don't include duplicate entries
+        if (list->contains(m_devices[n].name)) {
+            continue;
+        }
+
+        // Skip the default device, since we already added it
+        if (m_devices[n].name == defaultDeviceName
+            && m_devices[n].api == baseRtAudioApi) {
+            continue;
+        }
+
+        // Skip if no channels available
+        if ((isInput && m_devices[n].inputChannels == 0)
+            || (!isInput && m_devices[n].outputChannels == 0)) {
+            continue;
+        }
+
+        // Skip blacklisted devices
+        if (blacklisted_devices.contains(m_devices[n].name)) {
+            std::cout << "RTAudio: blacklisted " << (isInput ? "input" : "output")
+                      << " device: " << m_devices[n].name.toStdString() << std::endl;
+            continue;
+        }
+
+        // Good to go!
+        if (isInput) {
+            list->append(m_devices[n].name);
+            if (channels != NULL) {
+                channels->append(m_devices[n].inputChannels);
+            }
+        } else {
+            list->append(m_devices[n].name);
+            if (channels != NULL) {
+                channels->append(m_devices[n].outputChannels);
+            }
+        }
+
+        if (categories == NULL) {
+            continue;
+        }
+
+#ifdef _WIN32
+        switch (m_devices[n].api) {
+        case RtAudio::WINDOWS_ASIO:
+            categories->append("Low-Latency (ASIO)");
+            break;
+        case RtAudio::WINDOWS_WASAPI:
+            categories->append("High-Latency (Non-ASIO)");
+            break;
+        case RtAudio::WINDOWS_DS:
+            categories->append("High-Latency (Non-ASIO)");
+            break;
+        default:
+            categories->append("");
+            break;
+        }
+#else
+        categories->append("");
+#endif
+    }
+}
+#endif
+
 void VsAudioInterface::setAudioInterfaceMode(bool useRtAudio, bool shouldRestart)
 {
     if (useRtAudio) {
@@ -361,6 +599,7 @@ void VsAudioInterface::setupPlugins()
 {
     // Create plugins
     m_inputMeter         = new Meter(getNumInputChannels());
+    m_outputMeter        = new Meter(getNumOutputChannels());
     m_inputVolumePlugin  = new Volume(getNumInputChannels());
     m_outputVolumePlugin = new Volume(getNumOutputChannels());
     m_outputTonePlugin   = new Tone(getNumOutputChannels());
@@ -370,10 +609,13 @@ void VsAudioInterface::setupPlugins()
     addInputPlugin(m_inputVolumePlugin);
     addOutputPlugin(m_outputVolumePlugin);
     addInputPlugin(m_inputMeter);
+    addOutputPlugin(m_outputMeter);
 
     // Connect plugins for communication with UI
     connect(m_inputMeter, &Meter::onComputedVolumeMeasurements, this,
-            &VsAudioInterface::processMeterMeasurements);
+            &VsAudioInterface::processInputMeterMeasurements);
+    connect(m_outputMeter, &Meter::onComputedVolumeMeasurements, this,
+            &VsAudioInterface::processOutputMeterMeasurements);
     connect(this, &VsAudioInterface::updatedInputVolume, m_inputVolumePlugin,
             &Volume::volumeUpdated);
     connect(this, &VsAudioInterface::updatedOutputVolume, m_outputVolumePlugin,
