@@ -40,9 +40,8 @@
 #include <QDebug>
 
 // Constructor
-VsDevice::VsDevice(QOAuth2AuthorizationCodeFlow* authenticator, bool testMode,
-                   QObject* parent)
-    : QObject(parent), m_authenticator(authenticator)
+VsDevice::VsDevice(VsAuth* auth, VsApi* api, QObject* parent)
+    : QObject(parent), m_auth(auth), m_api(api), m_sendVolumeTimer(this)
 {
     QSettings settings;
     settings.beginGroup(QStringLiteral("VirtualStudio"));
@@ -58,17 +57,12 @@ VsDevice::VsDevice(QOAuth2AuthorizationCodeFlow* authenticator, bool testMode,
     m_playbackVolume =
         (float)settings.value(QStringLiteral("OutMultiplier"), 1.0).toDouble();
     m_playbackMute = settings.value(QStringLiteral("OutMuted"), false).toBool();
+    m_monitorVolume =
+        (float)settings.value(QStringLiteral("MonMultiplier"), 0).toDouble();
     settings.endGroup();
 
-    m_sendVolumeTimer = new QTimer(this);
-    m_sendVolumeTimer->setSingleShot(true);
-    connect(m_sendVolumeTimer, &QTimer::timeout, this, &VsDevice::sendLevels);
-
-    // Determine which API host to use
-    m_apiHost = PROD_API_HOST;
-    if (testMode) {
-        m_apiHost = TEST_API_HOST;
-    }
+    m_sendVolumeTimer.setSingleShot(true);
+    connect(&m_sendVolumeTimer, &QTimer::timeout, this, &VsDevice::sendLevels);
 
     // Set server levels to stored versions
     QJsonObject json = {
@@ -76,12 +70,10 @@ VsDevice::VsDevice(QOAuth2AuthorizationCodeFlow* authenticator, bool testMode,
         {QLatin1String("captureMute"), m_captureMute},
         {QLatin1String("playbackVolume"), (int)(m_playbackVolume * 100.0)},
         {QLatin1String("playbackMute"), m_playbackMute},
-    };
+        {QLatin1String("monitorVolume"), (int)(m_monitorVolume * 100.0)}};
     QJsonDocument request = QJsonDocument(json);
 
-    QNetworkReply* reply = m_authenticator->put(
-        QStringLiteral("https://%1/api/devices/%2").arg(m_apiHost, m_appID),
-        request.toJson());
+    QNetworkReply* reply = m_api->updateDevice(m_appID, request.toJson());
     connect(reply, &QNetworkReply::finished, this, [=]() {
         // Got error
         if (reply->error() != QNetworkReply::NoError) {
@@ -112,6 +104,12 @@ VsDevice::VsDevice(QOAuth2AuthorizationCodeFlow* authenticator, bool testMode,
                 deviceState.object()[QStringLiteral("playbackMute")].toBool();
             emit updatedPlaybackVolumeFromServer(m_playbackVolume);
             emit updatedPlaybackMuteFromServer(m_playbackMute);
+
+            // monitor volume
+            m_monitorVolume =
+                (float)(deviceState.object()[QStringLiteral("monitorVolume")].toDouble()
+                        / 100.0);
+            emit updatedMonitorVolume(m_monitorVolume);
         }
 
         QSettings settings;
@@ -120,12 +118,20 @@ VsDevice::VsDevice(QOAuth2AuthorizationCodeFlow* authenticator, bool testMode,
         settings.setValue(QStringLiteral("InMuted"), m_captureMute);
         settings.setValue(QStringLiteral("OutMultiplier"), m_playbackVolume);
         settings.setValue(QStringLiteral("OutMuted"), m_playbackMute);
+        settings.setValue(QStringLiteral("MonMultiplier"), m_monitorVolume);
         settings.endGroup();
 
         reply->deleteLater();
     });
 }
 
+VsDevice::~VsDevice()
+{
+    m_sendVolumeTimer.stop();
+    stopJackTrip();
+    stopPinger();
+}
+
 // registerApp idempotently registers an emulated device belonging to the current user
 void VsDevice::registerApp()
 {
@@ -134,8 +140,8 @@ void VsDevice::registerApp()
     }
 
     // check if device exists
-    QNetworkReply* reply = m_authenticator->get(
-        QStringLiteral("https://%1/api/devices/%2").arg(m_apiHost, m_appID));
+    QNetworkReply* reply = m_api->getDevice(m_appID);
+    ;
     connect(reply, &QNetworkReply::finished, this, [=]() {
         // Got error
         if (reply->error() != QNetworkReply::NoError) {
@@ -186,8 +192,7 @@ void VsDevice::removeApp()
         return;
     }
 
-    QNetworkReply* reply = m_authenticator->deleteResource(
-        QStringLiteral("https://%1/api/devices/%2").arg(m_apiHost, m_appID));
+    QNetworkReply* reply = m_api->deleteDevice(m_appID);
     connect(reply, &QNetworkReply::finished, this, [=]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
@@ -218,8 +223,8 @@ void VsDevice::sendHeartbeat()
     if (m_webSocket == nullptr) {
         m_webSocket =
             new VsWebSocket(QUrl(QStringLiteral("wss://%1/api/devices/%2/heartbeat")
-                                     .arg(m_apiHost, m_appID)),
-                            m_authenticator->token(), m_apiPrefix, m_apiSecret);
+                                     .arg(m_api->getApiHost(), m_appID)),
+                            m_auth->accessToken(), m_apiPrefix, m_apiSecret);
         connect(m_webSocket, &VsWebSocket::textMessageReceived, this,
                 &VsDevice::onTextMessageReceived);
     }
@@ -279,9 +284,7 @@ void VsDevice::sendHeartbeat()
         m_webSocket->sendMessage(request.toJson());
     } else {
         // Send heartbeat via POST API
-        QNetworkReply* reply = m_authenticator->post(
-            QStringLiteral("https://%1/api/devices/%2/heartbeat").arg(m_apiHost, m_appID),
-            request.toJson());
+        QNetworkReply* reply = m_api->postDeviceHeartbeat(m_appID, request.toJson());
         connect(reply, &QNetworkReply::finished, this, [=]() {
             if (reply->error() != QNetworkReply::NoError) {
                 std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
@@ -297,6 +300,31 @@ void VsDevice::sendHeartbeat()
     }
 }
 
+bool VsDevice::reconnect()
+{
+    return m_reconnect;
+}
+
+void VsDevice::setReconnect(bool reconnect)
+{
+    m_reconnect = reconnect;
+    if (reconnect) {
+        stopPinger();
+        if (m_webSocket != nullptr && m_webSocket->isValid()) {
+            m_webSocket->closeSocket();
+        }
+        if (!m_jackTrip.isNull()) {
+            m_jackTrip->stop();
+            m_jackTrip.reset();
+        }
+    }
+}
+
+bool VsDevice::hasTerminated()
+{
+    return m_jackTrip.isNull();
+}
+
 // setServerId updates the emulated device with the provided serverId
 void VsDevice::setServerId(QString serverId)
 {
@@ -304,15 +332,14 @@ void VsDevice::setServerId(QString serverId)
         {QLatin1String("serverId"), serverId},
     };
     QJsonDocument request = QJsonDocument(json);
-    QNetworkReply* reply  = m_authenticator->put(
-         QStringLiteral("https://%1/api/devices/%2").arg(m_apiHost, m_appID),
-         request.toJson());
+    QNetworkReply* reply  = m_api->updateDevice(m_appID, request.toJson());
     connect(reply, &QNetworkReply::finished, this, [=]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
             reply->deleteLater();
             return;
         }
+        m_deviceAgentConfig.insert("serverId", serverId);
         reply->deleteLater();
     });
 }
@@ -325,11 +352,9 @@ void VsDevice::sendLevels()
         {QLatin1String("captureMute"), m_captureMute},
         {QLatin1String("playbackVolume"), (int)(m_playbackVolume * 100.0)},
         {QLatin1String("playbackMute"), m_playbackMute},
-    };
+        {QLatin1String("monitorVolume"), (int)(m_monitorVolume * 100.0)}};
     QJsonDocument request = QJsonDocument(json);
-    QNetworkReply* reply  = m_authenticator->put(
-         QStringLiteral("https://%1/api/devices/%2").arg(m_apiHost, m_appID),
-         request.toJson());
+    QNetworkReply* reply  = m_api->updateDevice(m_appID, request.toJson());
     connect(reply, &QNetworkReply::finished, this, [=]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
@@ -341,18 +366,22 @@ void VsDevice::sendLevels()
 }
 
 // initJackTrip spawns a new jacktrip process with the desired settings
-JackTrip* VsDevice::initJackTrip([[maybe_unused]] bool useRtAudio,
-                                 [[maybe_unused]] std::string input,
-                                 [[maybe_unused]] std::string output,
-                                 [[maybe_unused]] int bufferSize,
-                                 [[maybe_unused]] int bufferStrategy,
-                                 VsServerInfo* studioInfo)
+JackTrip* VsDevice::initJackTrip(
+    [[maybe_unused]] bool useRtAudio, [[maybe_unused]] std::string input,
+    [[maybe_unused]] std::string output, [[maybe_unused]] int baseInputChannel,
+    [[maybe_unused]] int numChannelsIn, [[maybe_unused]] int baseOutputChannel,
+    [[maybe_unused]] int numChannelsOut, [[maybe_unused]] int inputMixMode,
+    [[maybe_unused]] int bufferSize, [[maybe_unused]] int bufferStrategy,
+    VsServerInfo* studioInfo)
 {
-    m_jackTrip.reset(new JackTrip(JackTrip::CLIENTTOPINGSERVER, JackTrip::UDP, 2, 2,
+    m_jackTrip.reset(
+        new JackTrip(JackTrip::CLIENTTOPINGSERVER, JackTrip::UDP, baseInputChannel,
+                     numChannelsIn, baseOutputChannel, numChannelsOut,
+                     static_cast<AudioInterface::inputMixModeT>(inputMixMode),
 #ifdef WAIR  // wair
-                                  0,
+                     0,
 #endif  // endwhere
-                                  4, 1));
+                     4, 1));
     m_jackTrip->setConnectDefaultAudioPorts(true);
 #ifdef RT_AUDIO
     if (useRtAudio) {
@@ -363,10 +392,15 @@ JackTrip* VsDevice::initJackTrip([[maybe_unused]] bool useRtAudio,
         m_jackTrip->setOutputDevice(output);
     }
 #endif
+    int bindPort = selectBindPort();
+    if (bindPort == 0) {
+        return 0;
+    }
+    m_jackTrip->setBindPorts(bindPort);
     m_jackTrip->setRemoteClientName(m_appID);
     // increment m_bufferStrategy by 1 for array-index mapping
     m_jackTrip->setBufferStrategy(bufferStrategy + 1);
-    if (bufferStrategy == 2) {
+    if (bufferStrategy == 2 || bufferStrategy == 3) {
         // use -q auto3 for loss concealment
         m_jackTrip->setBufferQueueLength(-3);
     } else {
@@ -401,8 +435,12 @@ void VsDevice::startJackTrip()
 void VsDevice::stopJackTrip()
 {
     if (!m_jackTrip.isNull()) {
+        if (m_webSocket != nullptr && m_webSocket->isValid()) {
+            m_webSocket->closeSocket();
+        }
         setServerId("");
         m_jackTrip->stop();
+        m_jackTrip.reset();
     }
 }
 
@@ -455,10 +493,7 @@ void VsDevice::updateCaptureVolume(float multiplier)
         return;
     }
     m_captureVolume = multiplier;
-
-    if (m_sendVolumeTimer) {
-        m_sendVolumeTimer->start(200);
-    }
+    m_sendVolumeTimer.start(100);
 }
 
 // updateCaptureMute sets VsDevice's capture (input) mute to the provided boolean
@@ -468,10 +503,7 @@ void VsDevice::updateCaptureMute(bool muted)
         return;
     }
     m_captureMute = muted;
-
-    if (m_sendVolumeTimer) {
-        m_sendVolumeTimer->start(200);
-    }
+    m_sendVolumeTimer.start(100);
 }
 
 // updatePlaybackVolume sets VsDevice's playback (output) volume to the provided float
@@ -481,10 +513,7 @@ void VsDevice::updatePlaybackVolume(float multiplier)
         return;
     }
     m_playbackVolume = multiplier;
-
-    if (m_sendVolumeTimer) {
-        m_sendVolumeTimer->start(200);
-    }
+    m_sendVolumeTimer.start(100);
 }
 
 // updatePlaybackMute sets VsDevice's playback (output) mute to the provided boolean
@@ -494,10 +523,17 @@ void VsDevice::updatePlaybackMute(bool muted)
         return;
     }
     m_playbackMute = muted;
+    m_sendVolumeTimer.start(100);
+}
 
-    if (m_sendVolumeTimer) {
-        m_sendVolumeTimer->start(200);
+// updateMonitorVolume sets VsDevice's monitor to the provided float
+void VsDevice::updateMonitorVolume(float multiplier)
+{
+    if (multiplier == m_monitorVolume) {
+        return;
     }
+    m_monitorVolume = multiplier;
+    m_sendVolumeTimer.start(100);
 }
 
 // terminateJackTrip is a slot intended to be triggered on jacktrip process signals
@@ -506,7 +542,9 @@ void VsDevice::terminateJackTrip()
     if (!enabled()) {
         setServerId("");
     }
-    m_jackTrip.reset();
+    if (!m_jackTrip.isNull()) {
+        m_jackTrip.reset();
+    }
 }
 
 // onTextMessageReceived is a slot intended to be triggered by new incoming WSS messages
@@ -550,6 +588,13 @@ void VsDevice::onTextMessageReceived(const QString& message)
         emit updatedPlaybackMuteFromServer(m_playbackMute);
     }
 
+    // monitor volume
+    newVolume = (float)(newState["monitorVolume"].toDouble() / 100.0);
+    if (newVolume != m_monitorVolume) {
+        m_monitorVolume = newVolume;
+        emit updatedMonitorVolume(m_monitorVolume);
+    }
+
     reconcileAgentConfig(newState);
 }
 
@@ -614,8 +659,7 @@ void VsDevice::registerJTAsDevice()
     };
     QJsonDocument request = QJsonDocument(json);
 
-    QNetworkReply* reply = m_authenticator->post(
-        QStringLiteral("https://%1/api/devices").arg(m_apiHost), request.toJson());
+    QNetworkReply* reply = m_api->postDevice(request.toJson());
     connect(reply, &QNetworkReply::finished, this, [=]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
@@ -662,3 +706,21 @@ QString VsDevice::randomString(int stringLength)
 
     return str;
 }
+
+// selectBindPort finds the next open bind port to use for jacktrip
+int VsDevice::selectBindPort()
+{
+    int candidate = gDefaultPort;
+    if (m_jackTrip.isNull()) {
+        return candidate;
+    }
+    int attempt = 0;
+    while (attempt <= 5000) {
+        candidate = QRandomGenerator::global()->bounded(gBindPortLow, gBindPortHigh + 1);
+        attempt++;
+        if (!m_jackTrip->checkIfPortIsBinded(candidate)) {
+            return candidate;
+        }
+    }
+    return 0;
+}
