@@ -40,7 +40,7 @@
 
 #include "AudioInterface.h"
 
-constexpr const char* const gVersion = "1.7.0";  ///< JackTrip version
+constexpr const char* const gVersion = "1.10.0";  ///< JackTrip version
 
 //*******************************************************************************
 /// \name Default Values
@@ -126,7 +126,8 @@ constexpr int gMaxRemoteNameLength           = 64;
 /// \name Global Functions
 
 #ifdef __APPLE__
-void setRealtimeProcessPriority(int bufferSize, int sampleRate);
+void setRealtimeProcessPriority(int bufferSize = gDefaultBufferSizeInSamples,
+                                int sampleRate = gDefaultSampleRate);
 #else
 void setRealtimeProcessPriority();
 #endif
